--[[
    Helper functions
]]--
--[[
Copyright (c) 2008-2015, LordFarlander
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
]]--

local MAJOR_VERSION = "LibLordFarlander-2.0"
local MINOR_VERSION = 20150701033005

local LibLordFarlander, oldminor = LibStub:NewLibrary( MAJOR_VERSION, MINOR_VERSION )
if( not LibLordFarlander ) then
    return
end--if

local TR = LibStub( "LibLordFarlander-TableRecycler-1.0", true )

LibLordFarlander.Kalimdor = 1
LibLordFarlander.EasternKingdoms = 2
LibLordFarlander.Outland = 3
LibLordFarlander.Northrend = 4
LibLordFarlander.Maelstrom = 5
LibLordFarlander.Pandaria = 6
LibLordFarlander.Draenor = 7
LibLordFarlander.ColdWeatherFlying = 54197
LibLordFarlander.FlightMastersLicence = 90267

function LibLordFarlander.PlayerItemCount( item )
    local _, itemLink = GetItemInfo( item )

    if( itemLink ~= nil ) then
        return GetItemCount( itemLink )
    end--if
    return 0
end--LibLordFarlander.PlayerItemCount( item )

-- Notes:
-- Gets an item string from a link
-- Arguments:
--     string - a link, can be any type of link
-- Returns:
-- * string - the item string from the link
function LibLordFarlander.GetItemStringFromLink( itemLink )
    if( type( itemLink ) == "string" ) then
        return itemLink:match( "^|c%x+|H(.+)|h%[.+%]" )
    end--if
    return nil
end--LibLordFarlander.GetItemStringFromLink( itemLink )

function LibLordFarlander.IsFlyableZone( zone, continent )
    if( ( type( zone ) ~= "string" ) or ( type( continent ) ~= "number" ) ) then
        return false
    end--if
    if( not LibLordFarlander.NotFlyableZones ) then
        LibLordFarlander.NotFlyableZones = {
            [LibLordFarlander.EasternKingdoms] = { -- Eastern Kingdoms
                [GetMapNameByID( 401 )] = 401, -- Alterac Valley
                [GetMapNameByID( 461 )] = 461, -- Arathi Basin
                [GetMapNameByID( 860 )] = 860, -- Silvershard Mines
                [GetMapNameByID( 789 )] = 789, -- Sunwell Plateau
                [GetMapNameByID( 736 )] = 736, -- The Battle for Gilneas
                [GetMapNameByID( 626 )] = 626, -- Twin Peaks
            },
            [LibLordFarlander.Kalimdor] = { -- Kalimdor
                [GetMapNameByID( 464 )] = 464, -- Azuremyst Isle
                [GetMapNameByID( 476 )] = 476, -- Bloodmyst Isle
                [GetMapNameByID( 775 )] = 775, -- Hyjal Summit
                [GetMapNameByID( 734 )] = 734, -- Old Hillsbrad Foothills
                [GetMapNameByID( 717 )] = 717, -- Ruins of Ahn'Qiraj
                [GetMapNameByID( 953 )] = 953, -- Siege of Orgrimmar
                [GetMapNameByID( 766 )] = 766, -- Temple of Ahn'Qiraj
                [GetMapNameByID( 733 )] = 733, -- The Black Morass
                [GetMapNameByID( 521 )] = 521, -- The Culling of Stratholme
                [GetMapNameByID( 471 )] = 471, -- The Exodar
                [GetMapNameByID( 443 )] = 443, -- Warsong Gulch
                [GetMapNameByID( 688 )] = 688, -- Blackfathom Deeps
                [GetMapNameByID( 699 )] = 699, -- Dire Maul
                [GetMapNameByID( 824 )] = 824, -- Dragon Soul
                [GetMapNameByID( 820 )] = 820, -- End Time
                [GetMapNameByID( 800 )] = 800, -- Firelands
                [GetMapNameByID( 759 )] = 759, -- Halls of Origination
                [GetMapNameByID( 747 )] = 747, -- Lost City of the Tol'vir
                [GetMapNameByID( 750 )] = 750, -- Maraudon
                [GetMapNameByID( 718 )] = 718, -- Onyxia's Lair
                [GetMapNameByID( 680 )] = 680, -- Ragefire Chasm
                [GetMapNameByID( 760 )] = 760, -- Razorfen Downs
                [GetMapNameByID( 761 )] = 761, -- Razorfen Kraul
                [GetMapNameByID( 769 )] = 769, -- The Vortex Pinnacle
                [GetMapNameByID( 773 )] = 773, -- Throne of the Four Winds
                [GetMapNameByID( 749 )] = 749, -- Wailing Caverns
                [GetMapNameByID( 816 )] = 816, -- Well of Eternity
                [GetMapNameByID( 686 )] = 686, -- Zul'Farrak
            },
            [LibLordFarlander.Outland] = { -- Outland
                [GetMapNameByID( 722 )] = 722, -- Auchenai Crypts
                [GetMapNameByID( 796 )] = 796, -- Black Temple
                [GetMapNameByID( 776 )] = 776, -- Gruul's Lair
                [GetMapNameByID( 797 )] = 797, -- Hellfire Ramparts
                [GetMapNameByID( 779 )] = 779, -- Magtheridon's Lair
                [GetMapNameByID( 732 )] = 732, -- Mana-Tombs
                [GetMapNameByID( 780 )] = 780, -- Serpentshrine Cavern
                [GetMapNameByID( 723 )] = 723, -- Sethekk Halls
                [GetMapNameByID( 724 )] = 724, -- Shadow Labyrinth
                [GetMapNameByID( 731 )] = 731, -- The Arcatraz
                [GetMapNameByID( 725 )] = 725, -- The Blood Furnace
                [GetMapNameByID( 729 )] = 729, -- The Botanica
                [GetMapNameByID( 782 )] = 782, -- The Eye
                [GetMapNameByID( 730 )] = 730, -- The Mechanar
                [GetMapNameByID( 710 )] = 710, -- The Shattered Halls
                [GetMapNameByID( 728 )] = 728, -- The Slave Pens
                [GetMapNameByID( 727 )] = 727, -- The Steamvault
                [GetMapNameByID( 726 )] = 726, -- The Underbog
            },
            [LibLordFarlander.Northrend] = { -- Northrend
                [GetMapNameByID( 512 )] = 512, -- Strand of the Ancients
                [GetMapNameByID( 540 )] = 540, -- Isle of Conquest
                [GetMapNameByID( 528 )] = 528, -- The Oculus
                [GetMapNameByID( 542 )] = 542, -- Trial of the Champion
                [GetMapNameByID( 543 )] = 543, -- Trial of the Crusader
                [GetMapNameByID( 529 )] = 529, -- Ulduar
                [GetMapNameByID( 522 )] = 522, -- Ahn'kahet: The Old Kingdom
                [GetMapNameByID( 533 )] = 533, -- Azjol-Nerub
                [GetMapNameByID( 534 )] = 534, -- Drak'Tharon Keep
                [GetMapNameByID( 530 )] = 530, -- Gundrak
                [GetMapNameByID( 525 )] = 525, -- Halls of Lightning
                [GetMapNameByID( 603 )] = 603, -- Halls of Reflection
                [GetMapNameByID( 526 )] = 526, -- Halls of Stone
                [GetMapNameByID( 819 )] = 819, -- Hour of Twilight
                [GetMapNameByID( 604 )] = 604, -- Icecrown Citadel
                [GetMapNameByID( 602 )] = 602, -- Pit of Saron
                [GetMapNameByID( 535 )] = 535, -- Naxxramas
                [GetMapNameByID( 527 )] = 527, -- The Eye of Eternity
                [GetMapNameByID( 601 )] = 601, -- The Forge of Souls
                [GetMapNameByID( 520 )] = 520, -- The Nexus
                [GetMapNameByID( 531 )] = 531, -- The Obsidian Sanctum
                [GetMapNameByID( 609 )] = 609, -- The Ruby Sanctum
                [GetMapNameByID( 536 )] = 536, -- The Violet Hold
                [GetMapNameByID( 543 )] = 543, -- Trial of the Crusader
                [GetMapNameByID( 523 )] = 523, -- Utgarde Keep
                [GetMapNameByID( 524 )] = 524, -- Utgarde Pinnacle
                [GetMapNameByID( 532 )] = 532, -- Vault of Archavon
            },
            [LibLordFarlander.Maelstrom] = { -- Maelstrom
                [GetMapNameByID( 605 )] = 605, -- Kezan
                [GetMapNameByID( 544 )] = 544, -- The Lost Isles
                [GetMapNameByID( 768 )] = 768, -- The Stonecore
            },
            [LibLordFarlander.Pandaria] = { -- Pandaria
                [GetMapNameByID( 929 )] = 929, -- Isle of Giants
                [GetMapNameByID( 928 )] = 928, -- Isle of Thunder
                [GetMapNameByID( 951 )] = 951, -- Timeless Isle
                [GetMapNameByID( 935 )] = 935, -- Deepwind Gorge
                [GetMapNameByID( 875 )] = 875, -- Gate of the Setting Sun
                [GetMapNameByID( 897 )] = 897, -- Heart of Fear
                [GetMapNameByID( 885 )] = 885, -- Mogu'Shan Palace
                [GetMapNameByID( 896 )] = 896, -- Mogu'shan Vaults
                [GetMapNameByID( 877 )] = 877, -- Shado-pan Monastery
                [GetMapNameByID( 887 )] = 887, -- Siege of Niuzao Temple
                [GetMapNameByID( 876 )] = 876, -- Stormstout Brewery
                [GetMapNameByID( 856 )] = 856, -- Temple of Kotmogu
                [GetMapNameByID( 867 )] = 867, -- Temple of the Jade Serpent
                [GetMapNameByID( 886 )] = 886, -- Terrace of Endless Spring
                [GetMapNameByID( 930 )] = 930, -- Throne of Thunder
            },
            [LibLordFarlander.Pandaria] = { -- Draenor
                [GetMapNameByID( 978 )] = 978, -- Ashran
                [GetMapNameByID( 941 )] = 941, -- Frostfire Ridge
                [GetMapNameByID( 976 )] = 976, -- Frostwall
                [GetMapNameByID( 949 )] = 949, -- Gorgrond
                [GetMapNameByID( 971 )] = 971, -- Lunarfall
                [GetMapNameByID( 950 )] = 950, -- Nagrand
                [GetMapNameByID( 947 )] = 947, -- Shadowmoon Valley
                [GetMapNameByID( 948 )] = 948, -- Spires of Arak
                [GetMapNameByID( 1009 )] = 1009, -- Stormshield
                [GetMapNameByID( 946 )] = 946, -- Talador
                [GetMapNameByID( 945 )] = 945, -- Tanaan Jungle
                [GetMapNameByID( 970 )] = 970, -- Tanaan Jungle - Assault on the Dark Portal
                [GetMapNameByID( 1011 )] = 1011, -- Warspear
                [GetMapNameByID( 984 )] = 984, -- Auchindoun
                [GetMapNameByID( 988 )] = 988, -- Blackrock Foundry
                [GetMapNameByID( 964 )] = 964, -- Bloodmaul Slag Mines
                [GetMapNameByID( 993 )] = 993, -- Grimrail Depot
                [GetMapNameByID( 994 )] = 994, -- Highmaul
                [GetMapNameByID( 987 )] = 987, -- Iron Docks
                [GetMapNameByID( 969 )] = 969, -- Shadowmoon Burial Grounds
                [GetMapNameByID( 989 )] = 989, -- Skyreach
                [GetMapNameByID( 1008 )] = 1008, -- The Everbloom
            },
        }
    end--if
    return LibLordFarlander.NotFlyableZones[continent] and ( not LibLordFarlander.NotFlyableZones[continent][zone] ) or false
end--LibLordFarlander.IsFlyableZone( zone )

function LibLordFarlander.IsUnderwaterZone( zone )
    return ( zone == GetMapNameByID( 614 ) ) or -- Abyssal Depths
           ( zone == GetMapNameByID( 610 ) ) or -- Kelp'thar Forest
           ( zone == GetMapNameByID( 615 ) ) or -- Shimmering Expanse
           ( zone == GetMapNameByID( 613 ) ) -- Vashj'ir
end--LibLordFarlander.IsUnderwaterZone( zone )

function LibLordFarlander.IsInUnderwaterZone()
    return LibLordFarlander.IsUnderwaterZone( GetRealZoneText() )
end--LibLordFarlander.IsInUnderwaterZone()

-- Notes:
-- Gets an ID from a string
-- Arguments:
--     string - a string
-- Returns:
-- * number - the ID from the string
function LibLordFarlander.GetItemIDFromString( itemID )
    if( type( itemID ) == "string" ) then
        return tonumber( itemID:match( "(%d+)" ) )
    end--if
    return nil
end--LibLordFarlander.GetItemIDFromString( itemID )

-- Notes:
-- Gets an ID from a link
-- Arguments:
--     string - a link, can be any type
-- Returns:
-- * number - the ID from the link
function LibLordFarlander.GetItemIDFromLink( itemLink )
    if( type( itemLink ) == "string" ) then
        return LibLordFarlander.GetItemIDFromString( LibLordFarlander.GetItemStringFromLink( itemLink ) )
    end--if
    return nil
end--LibLordFarlander.GetItemIDFromLink( itemLink )

-- Notes:
-- Gets an item string from an ID
-- Arguments:
--     number - an item ID
-- Returns:
-- * string - the string from the ID
function LibLordFarlander.GetItemString( itemID )
    return LibLordFarlander.GetItemStringFromLink( LibLordFarlander.GetItemLink( itemID ) )
end--LibLordFarlander.GetItemString( itemID )

-- Notes:
-- Gets an item link from and ID
-- Arguments:
--     number - an item ID
-- Returns:
-- * string - the link from the ID
function LibLordFarlander.GetItemLink( itemID )
    local _, link = GetItemInfo( itemID )

    return link
end--LibLordFarlander.GetItemLink( itemID )

-- Notes:
-- Checks to see if the player has a spell
-- Arguments:
--     number, string - A spell ID or spell link
-- Returns:
-- * boolean - whether or not the player has the spell
function LibLordFarlander.HasPlayerSpell( spell )
    local spellName = GetSpellInfo( spell )

    if( spellName ) then
        return GetSpellLink( spell ) == GetSpellLink( spellName )
    end
    return false
end--LibLordFarlander.PlayerHasSpell( spell )

-- Notes:
-- Debug message printing, suitable for including in your addon's class.
-- Set your addon's DEBUG member to true to print.
-- Arguments:
--     addon - The addon object
--     string - The message to print
function LibLordFarlander.dprint( addon, msg )
    if( addon.DEBUG ) then
        DEFAULT_CHAT_FRAME:AddMessage( msg, 1.0, 0.0, 0.0 )
    end--if
end--LibLordFarlander.dprint( addon, msg )

function LibLordFarlander.round( num, idp )
    local mult = 10 ^ ( idp or 0 )

    return math.floor( num * mult + 0.5 ) / mult
end--LibLordFarlander.round( num, idp )

function LibLordFarlander.GetTOCVersion()
    return tonumber( select( 4, GetBuildInfo() ) )
end--LibLordFarlander.GetTOCVersion()

function LibLordFarlander.GetMapZonesWithExtra( continent, bMountableOnly )
    -- TODO: Scenarios?
    local mapzones = TR:GetTable()

    if( continent == LibLordFarlander.EasternKingdoms ) then -- Eastern Kingdoms
        -- Main zones
        mapzones[614] = GetMapNameByID( 614 ) -- Abyssal Depths
        mapzones[16] = GetMapNameByID( 16 ) -- Arathi Highlands
        mapzones[17] = GetMapNameByID( 17 ) -- Badlands
        mapzones[19] = GetMapNameByID( 19 ) -- Blasted Lands
        mapzones[29] = GetMapNameByID( 29 ) -- Burning Steppes
        mapzones[866] = GetMapNameByID( 866 ) -- Coldridge Valley
        mapzones[32] = GetMapNameByID( 32 ) -- Deadwind Pass
        mapzones[892] = GetMapNameByID( 892 ) -- Deathknell
        mapzones[27] = GetMapNameByID( 27 ) -- Dun Morogh
        mapzones[34] = GetMapNameByID( 34 ) -- Duskwood
        mapzones[23] = GetMapNameByID( 23 ) -- Eastern Plaguelands
        mapzones[30] = GetMapNameByID( 30 ) -- Elwynn Forest
        mapzones[462] = GetMapNameByID( 462 ) -- Eversong Woods
        mapzones[463] = GetMapNameByID( 463 ) -- Ghostlands
        mapzones[545] = GetMapNameByID( 545 ) -- Gilneas
        mapzones[611] = GetMapNameByID( 611 ) -- Gilneas City
        mapzones[24] = GetMapNameByID( 24 ) -- Hillsbrad Foothills
        mapzones[341] = GetMapNameByID( 341 ) -- Ironforge
        mapzones[499] = GetMapNameByID( 499 ) -- Isle of Quel'Danas
        mapzones[610] = GetMapNameByID( 610 ) -- Kelp'thar Forest
        mapzones[35] = GetMapNameByID( 35 ) -- Loch Modan
        mapzones[895] = GetMapNameByID( 895 ) -- New Tinkertown
        mapzones[37] = GetMapNameByID( 37 ) -- Northern Stranglethorn
        mapzones[864] = GetMapNameByID( 864 ) -- Northshire
        mapzones[36] = GetMapNameByID( 36 ) -- Redridge Mountains
        mapzones[684] = GetMapNameByID( 684 ) -- Ruins of Gilneas
        mapzones[685] = GetMapNameByID( 685 ) -- Ruins of Gilneas City
        mapzones[28] = GetMapNameByID( 28 ) -- Searing Gorge
        mapzones[615] = GetMapNameByID( 615 ) -- Shimmering Expanse
        mapzones[480] = GetMapNameByID( 480 ) -- Silvermoon City
        mapzones[21] = GetMapNameByID( 21 ) -- Silverpine Forest
        mapzones[301] = GetMapNameByID( 301 ) -- Stormwind City
        mapzones[689] = GetMapNameByID( 689 ) -- Stranglethorn Vale
        mapzones[893] = GetMapNameByID( 893 ) -- Sunstrider Isle
        mapzones[38] = GetMapNameByID( 38 ) -- Swamp of Sorrows
        mapzones[673] = GetMapNameByID( 673 ) -- The Cape of Stranglethorn
        mapzones[26] = GetMapNameByID( 26 ) -- The Hinterlands
        mapzones[502] = GetMapNameByID( 502 ) -- The Scarlet Enclave
        mapzones[20] = GetMapNameByID( 20 ) -- Tirisfal Glades
        mapzones[708] = GetMapNameByID( 708 ) -- Tol Barad
        mapzones[709] = GetMapNameByID( 709 ) -- Tol Barad Peninsula
        mapzones[700] = GetMapNameByID( 700 ) -- Twilight Highlands
        mapzones[382] = GetMapNameByID( 382 ) -- Undercity
        mapzones[613] = GetMapNameByID( 613 ) -- Vashj'ir
        mapzones[22] = GetMapNameByID( 22 ) -- Western Plaguelands
        mapzones[39] = GetMapNameByID( 39 ) -- Westfall
        mapzones[40] = GetMapNameByID( 40 ) -- Wetlands
        -- Instances
        mapzones[401] = GetMapNameByID( 401 ) -- Alterac Valley
        mapzones[461] = GetMapNameByID( 461 ) -- Arathi Basin
        mapzones[860] = GetMapNameByID( 860 ) -- Silvershard Mines
        mapzones[789] = GetMapNameByID( 789 ) -- Sunwell Plateau
        mapzones[736] = GetMapNameByID( 736 ) -- The Battle for Gilneas
        mapzones[626] = GetMapNameByID( 626 ) -- Twin Peaks
        if( not bMountableOnly ) then
            mapzones[752] = GetMapNameByID( 752 ) -- Baradin Hold
            mapzones[755] = GetMapNameByID( 755 ) -- Blackwing Lair
            mapzones[753] = GetMapNameByID( 753 ) -- Blackrock Caverns
            mapzones[754] = GetMapNameByID( 754 ) -- Blackwing Descent
            mapzones[704] = GetMapNameByID( 704 ) -- Blackrock Depths
            mapzones[721] = GetMapNameByID( 721 ) -- Blackrock Spire
            mapzones[691] = GetMapNameByID( 691 ) -- Gnomeregan
            mapzones[757] = GetMapNameByID( 757 ) -- Grim Batol
            mapzones[799] = GetMapNameByID( 799 ) -- Karazhan
            mapzones[798] = GetMapNameByID( 798 ) -- Magisters' Terrace
            mapzones[696] = GetMapNameByID( 696 ) -- Molten Core
            mapzones[871] = GetMapNameByID( 871 ) -- Scarlet Halls
            mapzones[874] = GetMapNameByID( 874 ) -- Scarlet Monastery
            mapzones[898] = GetMapNameByID( 898 ) -- Scholomance
            mapzones[764] = GetMapNameByID( 764 ) -- Shadowfang Keep
            mapzones[765] = GetMapNameByID( 765 ) -- Stratholme
            mapzones[758] = GetMapNameByID( 758 ) -- The Bastion of Twilight
            mapzones[756] = GetMapNameByID( 756 ) -- The Deadmines
            mapzones[690] = GetMapNameByID( 690 ) -- The Stockade
            mapzones[687] = GetMapNameByID( 687 ) -- The Temple of Atal'Hakkar
            mapzones[767] = GetMapNameByID( 767 ) -- Throne of the Tides
            mapzones[692] = GetMapNameByID( 692 ) -- Uldaman
            mapzones[995] = GetMapNameByID( 995 ) -- Upper Blackrock Spire
            mapzones[781] = GetMapNameByID( 781 ) -- Zul'Aman
            mapzones[793] = GetMapNameByID( 793 ) -- Zul'Gurub
        end--if
    elseif( continent == LibLordFarlander.Kalimdor ) then -- Kalimdor
        -- Main zones
        mapzones[772] = GetMapNameByID( 772 ) -- Ahn'Qiraj: The Fallen Kingdom
        mapzones[894] = GetMapNameByID( 894 ) -- Ammen Vale
        mapzones[43] = GetMapNameByID( 43 ) -- Ashenvale
        mapzones[181] = GetMapNameByID( 181 ) -- Azshara
        mapzones[464] = GetMapNameByID( 464 ) -- Azuremyst Isle
        mapzones[476] = GetMapNameByID( 476 ) -- Bloodmyst Isle
        mapzones[890] = GetMapNameByID( 890 ) -- Camp Narache
        mapzones[42] = GetMapNameByID( 42 ) -- Darkshore
        mapzones[381] = GetMapNameByID( 381 ) -- Darnassus
        mapzones[101] = GetMapNameByID( 101 ) -- Desolace
        mapzones[4] = GetMapNameByID( 4 ) -- Durotar
        mapzones[141] = GetMapNameByID( 141 ) -- Dustwallow Marsh
        mapzones[891] = GetMapNameByID( 891 ) -- Echo Isles
        mapzones[182] = GetMapNameByID( 182 ) -- Felwood
        mapzones[121] = GetMapNameByID( 121 ) -- Feralas
        mapzones[795] = GetMapNameByID( 795 ) -- Molten Front
        mapzones[241] = GetMapNameByID( 241 ) -- Moonglade
        mapzones[606] = GetMapNameByID( 606 ) -- Mount Hyjal
        mapzones[9] = GetMapNameByID( 9 ) -- Mulgore
        mapzones[11] = GetMapNameByID( 11 ) -- Northern Barrens
        mapzones[321] = GetMapNameByID( 321 ) -- Orgrimmar
        mapzones[888] = GetMapNameByID( 888 ) -- Shadowglen
        mapzones[261] = GetMapNameByID( 261 ) -- Silithus
        mapzones[607] = GetMapNameByID( 607 ) -- Southern Barrens
        mapzones[81] = GetMapNameByID( 81 ) -- Stonetalon Mountains
        mapzones[161] = GetMapNameByID( 161 ) -- Tanaris
        mapzones[41] = GetMapNameByID( 41 ) -- Teldrassil
        mapzones[471] = GetMapNameByID( 471 ) -- The Exodar
        mapzones[61] = GetMapNameByID( 61 ) -- Thousand Needles
        mapzones[362] = GetMapNameByID( 362 ) -- Thunder Bluff
        mapzones[720] = GetMapNameByID( 720 ) -- Uldum
        mapzones[201] = GetMapNameByID( 201 ) -- Un'Goro Crater
        mapzones[889] = GetMapNameByID( 889 ) -- Valley of Trials
        mapzones[281] = GetMapNameByID( 281 ) -- Winterspring
        -- Instances
        mapzones[775] = GetMapNameByID( 775 ) -- Hyjal Summit
        mapzones[734] = GetMapNameByID( 734 ) -- Old Hillsbrad Foothills
        mapzones[717] = GetMapNameByID( 717 ) -- Ruins of Ahn'Qiraj
        mapzones[953] = GetMapNameByID( 953 ) -- Siege of Orgrimmar
        mapzones[766] = GetMapNameByID( 766 ) -- Temple of Ahn'Qiraj
        mapzones[733] = GetMapNameByID( 733 ) -- The Black Morass
        mapzones[521] = GetMapNameByID( 521 ) -- The Culling of Stratholme
        mapzones[443] = GetMapNameByID( 443 ) -- Warsong Gulch
        if( not bMountableOnly ) then
            mapzones[688] = GetMapNameByID( 688 ) -- Blackfathom Deeps
            mapzones[699] = GetMapNameByID( 699 ) -- Dire Maul
            mapzones[824] = GetMapNameByID( 824 ) -- Dragon Soul
            mapzones[820] = GetMapNameByID( 820 ) -- End Time
            mapzones[800] = GetMapNameByID( 800 ) -- Firelands
            mapzones[759] = GetMapNameByID( 759 ) -- Halls of Origination
            mapzones[747] = GetMapNameByID( 747 ) -- Lost City of the Tol'vir
            mapzones[750] = GetMapNameByID( 750 ) -- Maraudon
            mapzones[718] = GetMapNameByID( 718 ) -- Onyxia's Lair
            mapzones[680] = GetMapNameByID( 680 ) -- Ragefire Chasm
            mapzones[760] = GetMapNameByID( 760 ) -- Razorfen Downs
            mapzones[761] = GetMapNameByID( 761 ) -- Razorfen Kraul
            mapzones[769] = GetMapNameByID( 769 ) -- The Vortex Pinnacle
            mapzones[773] = GetMapNameByID( 773 ) -- Throne of the Four Winds
            mapzones[749] = GetMapNameByID( 749 ) -- Wailing Caverns
            mapzones[816] = GetMapNameByID( 816 ) -- Well of Eternity
            mapzones[686] = GetMapNameByID( 686 ) -- Zul'Farrak
        end--if
    elseif( continent == LibLordFarlander.Outland ) then -- Outland
        -- Main zones
        mapzones[465] = GetMapNameByID( 465 ) -- Hellfire Peninsula
        mapzones[475] = GetMapNameByID( 475 ) -- Blade's Edge Mountains
        mapzones[477] = GetMapNameByID( 477 ) -- Nagrand
        mapzones[479] = GetMapNameByID( 479 ) -- Netherstorm
        mapzones[473] = GetMapNameByID( 473 ) -- Shadowmoon Valley
        mapzones[481] = GetMapNameByID( 481 ) -- Shattrath City
        mapzones[478] = GetMapNameByID( 478 ) -- Terokkar Forest
        mapzones[467] = GetMapNameByID( 467 ) -- Zangarmarsh
        -- Instances
        mapzones[482] = GetMapNameByID( 482 ) -- Eye of the Storm
        if( not bMountableOnly ) then
            mapzones[722] = GetMapNameByID( 722 ) -- Auchenai Crypts
            mapzones[796] = GetMapNameByID( 796 ) -- Black Temple
            mapzones[776] = GetMapNameByID( 776 ) -- Gruul's Lair
            mapzones[797] = GetMapNameByID( 797 ) -- Hellfire Ramparts
            mapzones[779] = GetMapNameByID( 779 ) -- Magtheridon's Lair
            mapzones[732] = GetMapNameByID( 732 ) -- Mana-Tombs
            mapzones[780] = GetMapNameByID( 780 ) -- Serpentshrine Cavern
            mapzones[723] = GetMapNameByID( 723 ) -- Sethekk Halls
            mapzones[724] = GetMapNameByID( 724 ) -- Shadow Labyrinth
            mapzones[731] = GetMapNameByID( 731 ) -- The Arcatraz
            mapzones[725] = GetMapNameByID( 725 ) -- The Blood Furnace
            mapzones[729] = GetMapNameByID( 729 ) -- The Botanica
            mapzones[782] = GetMapNameByID( 782 ) -- The Eye
            mapzones[730] = GetMapNameByID( 730 ) -- The Mechanar
            mapzones[710] = GetMapNameByID( 710 ) -- The Shattered Halls
            mapzones[728] = GetMapNameByID( 728 ) -- The Slave Pens
            mapzones[727] = GetMapNameByID( 727 ) -- The Steamvault
            mapzones[726] = GetMapNameByID( 726 ) -- The Underbog
        end--if
    elseif( continent == LibLordFarlander.Northrend ) then -- Northrend
        -- Main zones
        mapzones[486] = GetMapNameByID( 486 ) -- Borean Tundra
        mapzones[510] = GetMapNameByID( 510 ) -- Crystalsong Forest
        mapzones[504] = GetMapNameByID( 504 ) -- Dalaran
        mapzones[488] = GetMapNameByID( 488 ) -- Dragonblight
        mapzones[490] = GetMapNameByID( 490 ) -- Grizzly Hills
        mapzones[491] = GetMapNameByID( 491 ) -- Howling Fjord
        mapzones[541] = GetMapNameByID( 541 ) -- Hrothgar's Landing
        mapzones[492] = GetMapNameByID( 492 ) -- Icecrown
        mapzones[493] = GetMapNameByID( 493 ) -- Sholazar Basin
        mapzones[495] = GetMapNameByID( 495 ) -- The Storm Peaks
        mapzones[501] = GetMapNameByID( 501 ) -- Wintergrasp
        mapzones[496] = GetMapNameByID( 496 ) -- Zul'Drak
        -- Instances
        mapzones[512] = GetMapNameByID( 512 ) -- Strand of the Ancients
        mapzones[540] = GetMapNameByID( 540 ) -- Isle of Conquest
        mapzones[528] = GetMapNameByID( 528 ) -- The Oculus
        mapzones[542] = GetMapNameByID( 542 ) -- Trial of the Champion
        mapzones[543] = GetMapNameByID( 543 ) -- Trial of the Crusader
        mapzones[529] = GetMapNameByID( 529 ) -- Ulduar
        if( not bMountableOnly ) then
            mapzones[522] = GetMapNameByID( 522 ) -- Ahn'kahet: The Old Kingdom
            mapzones[533] = GetMapNameByID( 533 ) -- Azjol-Nerub
            mapzones[534] = GetMapNameByID( 534 ) -- Drak'Tharon Keep
            mapzones[530] = GetMapNameByID( 530 ) -- Gundrak
            mapzones[525] = GetMapNameByID( 525 ) -- Halls of Lightning
            mapzones[603] = GetMapNameByID( 603 ) -- Halls of Reflection
            mapzones[526] = GetMapNameByID( 526 ) -- Halls of Stone
            mapzones[819] = GetMapNameByID( 819 ) -- Hour of Twilight
            mapzones[604] = GetMapNameByID( 604 ) -- Icecrown Citadel
            mapzones[602] = GetMapNameByID( 602 ) -- Pit of Saron
            mapzones[535] = GetMapNameByID( 535 ) -- Naxxramas
            mapzones[527] = GetMapNameByID( 527 ) -- The Eye of Eternity
            mapzones[601] = GetMapNameByID( 601 ) -- The Forge of Souls
            mapzones[520] = GetMapNameByID( 520 ) -- The Nexus
            mapzones[531] = GetMapNameByID( 531 ) -- The Obsidian Sanctum
            mapzones[609] = GetMapNameByID( 609 ) -- The Ruby Sanctum
            mapzones[536] = GetMapNameByID( 536 ) -- The Violet Hold
            mapzones[543] = GetMapNameByID( 543 ) -- Trial of the Crusader
            mapzones[523] = GetMapNameByID( 523 ) -- Utgarde Keep
            mapzones[524] = GetMapNameByID( 524 ) -- Utgarde Pinnacle
            mapzones[532] = GetMapNameByID( 532 ) -- Vault of Archavon
        end--if
    elseif( continent == LibLordFarlander.Maelstrom ) then -- Maelstrom
        -- Main zones
        mapzones[640] = GetMapNameByID( 640 ) -- Deepholm
        mapzones[605] = GetMapNameByID( 605 ) -- Kezan
        mapzones[544] = GetMapNameByID( 544 ) -- The Lost Isles
        if( not bMountableOnly ) then
            mapzones[768] = GetMapNameByID( 768 ) -- The Stonecore
        end--if
    elseif( continent == LibLordFarlander.Pandaria ) then -- Pandaria
        -- Main zones
        mapzones[858] = GetMapNameByID( 858 ) -- Dread Wastes
        mapzones[929] = GetMapNameByID( 929 ) -- Isle of Giants
        mapzones[928] = GetMapNameByID( 928 ) -- Isle of Thunder
        mapzones[857] = GetMapNameByID( 857 ) -- Krasarang Wilds
        mapzones[809] = GetMapNameByID( 809 ) -- Kun-Lai Summit
        mapzones[905] = GetMapNameByID( 905 ) -- Shrine of Seven Stars
        mapzones[903] = GetMapNameByID( 903 ) -- Shrine of Two Moons
        mapzones[806] = GetMapNameByID( 806 ) -- The Jade Forest
        mapzones[873] = GetMapNameByID( 873 ) -- The Veiled Stair
        mapzones[808] = GetMapNameByID( 808 ) -- The Wandering Isle
        mapzones[951] = GetMapNameByID( 951 ) -- Timeless Isle
        mapzones[810] = GetMapNameByID( 810 ) -- Townlong Steppes
        mapzones[811] = GetMapNameByID( 811 ) -- Vale of Eternal Blossoms
        mapzones[807] = GetMapNameByID( 807 ) -- Valley of the Four Winds
        -- Instances
        if( not bMountableOnly ) then
            mapzones[935] = GetMapNameByID( 935 ) -- Deepwind Gorge
            mapzones[875] = GetMapNameByID( 875 ) -- Gate of the Setting Sun
            mapzones[897] = GetMapNameByID( 897 ) -- Heart of Fear
            mapzones[885] = GetMapNameByID( 885 ) -- Mogu'Shan Palace
            mapzones[896] = GetMapNameByID( 896 ) -- Mogu'shan Vaults
            mapzones[877] = GetMapNameByID( 877 ) -- Shado-pan Monastery
            mapzones[887] = GetMapNameByID( 887 ) -- Siege of Niuzao Temple
            mapzones[876] = GetMapNameByID( 876 ) -- Stormstout Brewery
            mapzones[856] = GetMapNameByID( 856 ) -- Temple of Kotmogu
            mapzones[867] = GetMapNameByID( 867 ) -- Temple of the Jade Serpent
            mapzones[886] = GetMapNameByID( 886 ) -- Terrace of Endless Spring
            mapzones[930] = GetMapNameByID( 930 ) -- Throne of Thunder
        end--if
    elseif( continent == LibLordFarlander.Draenor ) then -- Draenor
        -- Main zones
        mapzones[978] = GetMapNameByID( 978 ) -- Ashran
        mapzones[941] = GetMapNameByID( 941 ) -- Frostfire Ridge
        mapzones[976] = GetMapNameByID( 976 ) -- Frostwall
        mapzones[949] = GetMapNameByID( 949 ) -- Gorgrond
        mapzones[971] = GetMapNameByID( 971 ) -- Lunarfall
        mapzones[950] = GetMapNameByID( 950 ) -- Nagrand
        mapzones[947] = GetMapNameByID( 947 ) -- Shadowmoon Valley
        mapzones[948] = GetMapNameByID( 948 ) -- Spires of Arak
        mapzones[1009] = GetMapNameByID( 1009 ) -- Stormshield
        mapzones[946] = GetMapNameByID( 946 ) -- Talador
        mapzones[945] = GetMapNameByID( 945 ) -- Tanaan Jungle
        mapzones[970] = GetMapNameByID( 970 ) -- Tanaan Jungle - Assault on the Dark Portal
        mapzones[1011] = GetMapNameByID( 1011 ) -- Warspear
        if( not bMountableOnly ) then
            mapzones[984] = GetMapNameByID( 984 ) -- Auchindoun
            mapzones[988] = GetMapNameByID( 988 ) -- Blackrock Foundry
            mapzones[964] = GetMapNameByID( 964 ) -- Bloodmaul Slag Mines
            mapzones[993] = GetMapNameByID( 993 ) -- Grimrail Depot
            mapzones[994] = GetMapNameByID( 994 ) -- Highmaul
            mapzones[987] = GetMapNameByID( 987 ) -- Iron Docks
            mapzones[969] = GetMapNameByID( 969 ) -- Shadowmoon Burial Grounds
            mapzones[989] = GetMapNameByID( 989 ) -- Skyreach
            mapzones[1008] = GetMapNameByID( 1008 ) -- The Everbloom
        end--if
    end--if
    return mapzones
end--LibLordFarlander.GetMapZonesWithExtra( continent )

function LibLordFarlander.NamifyString( str )
    return str:gsub( "([%c%s%p])", function( input ) return ("_%03i"):format( strbyte( input ) ) end )
end--LibLordFarlander.NamifyString( str )

function LibLordFarlander.UnnamifyString( str )
    return str:gsub( "_(%d%d%d)", function( input ) return strchar( tonumber( input ) ) end )
end--LibLordFarladner.UnnamifyString( str )
